#!/usr/bin/perl
#
# seq - procedures for reading a file of DNA sequences.

use 5.6.0;
use strict;
use warnings;


# Read in the sequence from a sequence file.

sub seq_get {

    (local *SEQ_FILE, my $seq_file) = @_;	# The sequence handle and file.
    my @seq = ();				# The genomic sequence.
    my $header = 0;				# Have we read in a sequence header?

    while (<SEQ_FILE>) {
	next if (/^\s*$/);
        if (/^\s*>/) {
            if ($header) {
                warn "Warning: $0 does not support multiple contigs\n" ,
                     "in the sequence file -- only the first was used.\n";
                last;
            }
            $header = 1;
            next;
        }
        for (split(//)) {
            next if (/\s/);
            die "$0: Non-DNA character '$_' in sequence \"$seq_file\".\n"
                unless (char_is_good($_));
            push (@seq, $_);
        }
    }
    return \@seq;


    # Is the char a valid sequence char?

    sub char_is_good {
        return (/[ABCDGHKMNRSTVWXY]/i);
    }

}


# Compute the reverse complement of the given sequence.

sub do_revcomp {

    my ($seq_file, $seq) = @_;		# Name of sequence file, sequence.

    foreach (@$seq) {
	my $char = $_;
        die "$0: Non-DNA character '$char' in sequence \"$seq_file\".\n"
	    unless defined($_ = get_comp($_));
    }
    @$seq = reverse(@$seq);
    return $seq;



    # Compute complement of given char, preserving all whitespaces.

    sub get_comp {

    	my %rev_hash = ("A", "T",           "a", "t",
       	                "B", "V",           "b", "v",
                        "C", "G",           "c", "g",
                        "D", "H",           "d", "h",
                        "G", "C",           "g", "c",
                        "H", "D",           "h", "d",
                        "K", "M",           "k", "m",
                        "M", "K",           "m", "k",
                        "N", "N",           "n", "n",
                        "R", "Y",           "r", "y",
                        "S", "S",           "s", "s",
                        "T", "A",           "t", "a",
                        "V", "B",           "v", "b",
                        "W", "W",           "w", "w",
                        "X", "X",           "x", "x",
                        "Y", "R",           "y", "r",);

	(exists($rev_hash{$_[0]}) ? return $rev_hash{$_[0]} : return);
    }
}



return 1;
